@@ -903,7 +903,7 @@ impl<'lock> PylockToml {
         let mut doc = toml_edit::DocumentMut::new();
 
         doc.insert("lock-version", value(self.lock_version.to_string()));
-        doc.insert("created-by", value(self.created_by.to_string()));
+        doc.insert("created-by", value(self.created_by.as_str()));
         if let Some(ref requires_python) = self.requires_python {
             doc.insert("requires-python", value(requires_python.to_string()));
         }
@@ -1420,7 +1420,7 @@ impl PylockTomlVcs {
             let mut url = if let Some(url) = self.url.as_ref() {
                 url.clone()
             } else if let Some(path) = self.path.as_ref() {
-                DisplaySafeUrl::from(
+                DisplaySafeUrl::from_url(
                     Url::from_directory_path(install_path.join(path))
                         .map_err(|()| PylockTomlErrorKind::PathToUrl)?,
                 )
